
/*
    VPPP virtual PPP connection over TCP.

    Copyright (C) 1998  Maxim Krasnyansky <max@uznet.net>

    Idea is based on Sam Lantinga's PPPTCP package, but code was 
    completely written from the scratch.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

/*
 * Version: 1.2 12/01/1998 Maxim Krasnyansky <max@uznet.net>
 */ 

#include <fcntl.h>
#include <stdlib.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <strings.h>
#include <stdarg.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <sys/mman.h>
#include <sched.h>

#include "vppp.h"

extern char *proc_name;

extern int errno;

/* Link two file descriptors on the specified speed */

int link_fd_err = 0;
long  link_fd_len = 0; 
int link_fd_fd1 = 0;
int link_fd_fd2 = 0;
int link_fd_delta1 = 0;
int link_fd_delta2 = 0;

int link_fd_maxfd  = 0;
fd_set link_fd_set;
struct timeval link_fd_tv={0,0};
     
char * link_fd_buf;

void link_fd_copy(int sig){
     register int len;

     FD_ZERO(&link_fd_set);
     FD_SET(link_fd_fd1, &link_fd_set);
     FD_SET(link_fd_fd2, &link_fd_set);

     errno = 0;

     /* Check for available data, or we will sleep in one 
      * of the read calls */
     if( ! select(link_fd_maxfd, &link_fd_set, NULL, NULL, &link_fd_tv) ){
	link_fd_delta1 = link_fd_delta2 = link_fd_len;
	return;
     }

     if( FD_ISSET(link_fd_fd1, &link_fd_set) ){
       if( (len=read(link_fd_fd1, link_fd_buf, link_fd_len+link_fd_delta1))<= 0)
	  link_fd_err=1;
       else 
  	  if( write_n(link_fd_fd2, link_fd_buf, len) < 0 )
	     link_fd_err=1; 
       link_fd_delta1 = 0; 

     }

     if( FD_ISSET(link_fd_fd2, &link_fd_set) ){
       if((len=read(link_fd_fd2, link_fd_buf, link_fd_len+link_fd_delta2)) <= 0)
	  link_fd_err=1;
       else 
	  if( write_n(link_fd_fd1, link_fd_buf, len) < 0 )
	     link_fd_err=1;
       link_fd_delta2 = 0; 
     }
}

int link_fd_shaper(int fd1, int fd2, int speed){
     struct itimerval itm;
     struct sigaction sa;
     int old_prio;

     if(speed < 64)
	  return 1;

     /* Calculate how much we will copy each timer tick 
      * It is very primitive shaping, but shaping results 
      * is prety accurate.
      * 85 is the magic number :)) for 64Kbit/sec. 
      */
 
     link_fd_len=85 * (speed / 64);

     if( !(link_fd_buf=malloc(link_fd_len*2)) )
         return 1; 
	
     link_fd_fd1 = fd1;
     link_fd_fd2 = fd2;
     link_fd_delta1 = link_fd_delta2 = link_fd_err = 0;

     link_fd_maxfd = (fd1 > fd2 ? fd1 : fd2) + 1;    
 
     /* Set up the SIGALRM handler */
     memset(&sa,0,sizeof(sa));    
     sa.sa_handler=link_fd_copy;
     sigemptyset(&sa.sa_mask);
     sigfillset(&sa.sa_mask);
     sigaction(SIGALRM,&sa,NULL);    

     /* Increase our priority and lock buf memmory */
     mlock(link_fd_buf,link_fd_len*2);
     old_prio=getpriority(PRIO_PROCESS,0);
     setpriority(PRIO_PROCESS,0,VPPP_LINKFD_PRIO);

     /* Start the timer */
     memset(&itm,0,sizeof(itm));
     itm.it_interval.tv_usec = itm.it_value.tv_usec = 1000;
     setitimer(ITIMER_REAL,&itm,NULL);
     
     while(!link_fd_err)
        pause();

     signal(SIGALRM,SIG_DFL);  
 
     setpriority(PRIO_PROCESS,0,old_prio);
     munlock(link_fd_buf,link_fd_len*2);

     memset(&itm,0,sizeof(itm));
     setitimer(ITIMER_REAL,&itm,NULL);

     free(link_fd_buf);
	
     return 0;
}

int link_fd(int fd1,int fd2){
	register int maxfd,len;
	int old_prio;
	fd_set fdset;

	if( !(link_fd_buf=malloc(VPPP_BUF_SIZE)) )
	  return 1; 

	maxfd = (fd1 > fd2 ? fd1 : fd2) + 1;

        /* Increase our priority and lock buf memmory */
        mlock(link_fd_buf,VPPP_BUF_SIZE);
        old_prio=getpriority(PRIO_PROCESS,0);
        setpriority(PRIO_PROCESS,0,VPPP_LINKFD_PRIO);

	while(1){
	   /* Wait for data */
	   FD_ZERO(&fdset);
	   FD_SET(fd1, &fdset);
	   FD_SET(fd2, &fdset);
	   if(select(maxfd, &fdset, NULL, NULL, NULL) < 0 )
	 	break;

	   if( FD_ISSET(fd1, &fdset) ){
	 	if( (len=read(fd1, link_fd_buf, VPPP_BUF_SIZE)) <= 0 )
			break;  
		if( write_n(fd2, link_fd_buf, len) < 0 )
			break; 
	   }
	   if( FD_ISSET(fd2, &fdset) ){
	 	if( (len=read(fd2, link_fd_buf, VPPP_BUF_SIZE)) <= 0 )
			break;
		if( write_n(fd1, link_fd_buf, len) < 0 )
			break;
	   }
	}
	setpriority(PRIO_PROCESS,0,old_prio);
        munlock(link_fd_buf,VPPP_BUF_SIZE);

	free(link_fd_buf);

	return 0;
}


