/*  
    VPPP virtual PPP connection over TCP.

    Copyright (C) 1998  Maxim Krasnyansky <max@uznet.net>

    Idea is based on Sam Lantinga's PPPTCP package, but code was 
    completely written from the scratch.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

/*
 * Version: 1.2 12/01/1998 Maxim Krasnyansky <max@uznet.net>
 */ 

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>

#include "vppp.h"

int port=VPPP_PORT;
int conn_timeout=VPPP_CONN_TIMEOUT;

extern char *optarg;
extern int optind,opterr,optopt;

extern int h_errno;

char *proc_name="vppp";

int main(int argc,char *argv[],char *env[]){
     int s;
     int persist=0,opt;	
     struct sockaddr_in my_addr,svr_addr;
     char *pppd_args=NULL;
     char *svr=NULL;
     char mesg[VPPP_MESG_SIZE];
     unsigned int speed=0;
     struct hostent *hent;
     struct sigaction sa;

     while( (opt=getopt(argc,argv,"po:P:t:")) != EOF ){
	switch(opt){
	    case 'p':
		persist=1;
		break;
	    case 'o':
	        pppd_args=strdup(optarg);
		break;
	    case 'P':
	        port=atoi(optarg);	
	        break;
	    case 't':
	        conn_timeout=atoi(optarg);	
	        break;
	    default:
	        printf("Usage:vppp [-p] [-P port] [-t timeout] [-o \"pppd options\"] <server>\n");
	        exit(1);
	}
     }	

     if( !argv[optind] ){
	printf("Usage:vppp [-p] [-P port] [-t timeout] [-o \"pppd options\"] <server>\n");
	exit(1);
     }   	

     /* Save server name since we will modify argv array */
     svr=strdup(argv[optind]);

     /* Become daemon */
     if(fork())
     	exit(0);

     close(0);close(1);close(2);
     setsid(); 
     
     memset(&sa,0,sizeof(sa));     
     sa.sa_handler=SIG_IGN;
     sigaction(SIGHUP,&sa,NULL);
     sigaction(SIGINT,&sa,NULL);
     sigaction(SIGQUIT,&sa,NULL);
     sigaction(SIGCHLD,&sa,NULL);
     sigaction(SIGPIPE,&sa,NULL);
     init_title(argc,argv,env);

     openlog("vppp",LOG_PID,LOG_DAEMON);

     if( !(hent=gethostbyname(svr)) ){
	  syslog(LOG_ERR,"%s",hstrerror(h_errno));
	  exit(1);
     }
	
     memset(&my_addr,0,sizeof(my_addr));
     my_addr.sin_family = AF_INET;
     my_addr.sin_addr.s_addr = INADDR_ANY;
     my_addr.sin_port = 0;	

     memset(&svr_addr,0,sizeof(my_addr));
     svr_addr.sin_family = AF_INET;
     svr_addr.sin_addr.s_addr = *((unsigned long *)hent->h_addr);
     svr_addr.sin_port = htons(port);	

     while(1){ 
	/* We have to create socket again each time
	 * we want to connect, since STREAM sockets 
	 * can be successfuly connected only once.
	 */
	
        if( (s=socket(AF_INET,SOCK_STREAM,0))==-1 ){
	  syslog(LOG_ERR,"Can not create socket");
	  exit(1);
        }

        if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
	  syslog(LOG_ERR,"Can not bind to the socket");
	  exit(1);
        }

	set_title("%s: connecting to %s",proc_name,svr);

        if( connect_t(s,(struct sockaddr *) &svr_addr, conn_timeout) ){
	  syslog(LOG_INFO,"Can't connect to the server %s",svr);
        } else {

	  if( read_n(s,mesg,VPPP_MESG_SIZE) < 0){   
	     syslog(LOG_INFO,"Error reading server mesg",svr);
	  } else {    
             if(mesg[0] == '1'){ 
	        speed=atoi(mesg+1);

	        syslog(LOG_INFO,"PPP session with %s speed %d",svr,speed);

	        opt=1;
		setsockopt(s,IPPROTO_TCP,TCP_NODELAY,&opt,sizeof(opt)); 

                start_ppp(s,svr,speed,pppd_args);

	        syslog(LOG_INFO,"PPP session with %s closed",svr);
	     } else {
	        syslog(LOG_INFO,"Connection dennied by server",svr);
	     }
	  }   
	}

	close(s);
	
	/* If persist option is set, sleep 
	 * and try to connect again */
	if(!persist) 
	  break;
        else
	  sleep(5);
     }
     syslog(LOG_INFO,"Exit");
     return 0;
}	
