
/*
    VPPP virtual PPP connection over TCP.

    Copyright (C) 1998  Maxim Krasnyansky <max@uznet.net>

    Idea is based on Sam Lantinga's PPPTCP package, but code was 
    completely written from the scratch.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

/*
 * Version: 1.2 12/01/1998 Maxim Krasnyansky <max@uznet.net>
 */ 

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <signal.h>
#include <syslog.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include "vppp.h"

struct vppp_host{
   char ip[16];
   unsigned int speed;
   char pppd_args[PPPD_ARGS_LEN];
} hosts[VPPP_MAX_HOSTS]= { { "",0,"" }, };

char *proc_name="vpppd";

char *hosts_file=VPPP_HOSTS_FILE;
int hosts_num=0;

int port=VPPP_PORT;

extern char *optarg;
extern int optind,opterr,optopt;

int read_hosts(void);
void sighup(int sig);

int main(int argc, char *argv[], char *env[]){
     int s,rt_fd;
     int l,f,opt;
     char *ip;
     char mesg[VPPP_MESG_SIZE];
     struct sigaction sa;
     struct sockaddr_in my_addr,cl_addr;

     while( (opt=getopt(argc,argv,"f:P:")) != EOF ){
	switch(opt){
	    case 'P':
		port=atoi(optarg);
		break;
	    case 'f':
		hosts_file=optarg;
		break;
	    default:
		printf("Usage: vpppd [-P port] [-f hosts_file]\n");
	        exit(1);
	}
     }	

     /* Become daemon */ 
     if( fork() )
	exit(0);

     close(0);close(1);close(2);
     setsid();

     openlog("vpppd",LOG_PID,LOG_DAEMON);
    

     switch( (hosts_num=read_hosts()) ){
	case 0:
	   syslog(LOG_ERR,"No hosts specified in %s",hosts_file);
	   exit(1);
	case -1:
	   syslog(LOG_ERR,"Error reading %s",hosts_file);
	   exit(1);
     }	
	
     memset(&sa,0,sizeof(sa));     
     sa.sa_handler=sighup;
     sigaction(SIGHUP,&sa,NULL);

     sa.sa_handler=SIG_IGN;
     sigaction(SIGINT,&sa,NULL);
     sigaction(SIGQUIT,&sa,NULL);
     sigaction(SIGCHLD,&sa,NULL);
     sigaction(SIGPIPE,&sa,NULL);

     memset(&my_addr,0,sizeof(my_addr));
     my_addr.sin_family = AF_INET;
     my_addr.sin_addr.s_addr = htonl(INADDR_ANY);
     my_addr.sin_port = htons(port);	

     if( (s=socket(AF_INET,SOCK_STREAM,0))== -1 ){
	syslog(LOG_ERR,"Can't creat socket");
	exit(1);
     }

     l=1;
     setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &l, sizeof(l)); 

     if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
	syslog(LOG_ERR,"Can't bind to the socket");
	exit(1);
     }

     if( listen(s,5) ){
	syslog(LOG_ERR,"Can't listen on the socket");
	exit(1);
     }

     init_title(argc,argv,env);
     set_title("%s: waiting for connections",proc_name);

     while(1){
        l=sizeof(cl_addr);
	if( (rt_fd=accept(s,&cl_addr,&l)) == -1 )
	   continue; 
	ip=inet_ntoa(cl_addr.sin_addr);
	
	/* Try to find IP in our hosts table */
	for(f=l=0;l<hosts_num;l++){
	   if(!strcmp(hosts[l].ip,ip)){
	     f=1; break;	
	   }
	}
	memset(mesg,0,VPPP_MESG_SIZE);
	switch(f){
	   case 1:
	     syslog(LOG_INFO,"Allowed %s <%s>",ip,hosts[l].pppd_args);
	     sprintf(mesg,"1 %d",hosts[l].speed); 
	     write_n(rt_fd,mesg,VPPP_MESG_SIZE);
	     break;
	   case 0:
	     syslog(LOG_INFO,"Denied %s",ip);
	     sprintf(mesg,"0"); 
	     write_n(rt_fd,mesg,VPPP_MESG_SIZE);
	     close(rt_fd);
	     continue;
	}     

	if( !fork() ){
	   syslog(LOG_INFO,"PPP session with %s speed %d",ip,hosts[l].speed);

           sa.sa_handler=SIG_IGN;
           sigaction(SIGHUP,&sa,NULL);

	   opt=1;
	   setsockopt(rt_fd,IPPROTO_TCP,TCP_NODELAY,&opt,sizeof(opt));

	   start_ppp(rt_fd,ip,hosts[l].speed,hosts[l].pppd_args);
	   close(rt_fd);
	   syslog(LOG_INFO,"PPP session with %s closed",ip);
	   exit(0);
        }
	close(rt_fd);
     } 
}	

void sighup(int sig)
{
     switch( (hosts_num=read_hosts()) ){
	case 0:
	   syslog(LOG_ERR,"No hosts specified in %s",hosts_file);
	   exit(1);
	case -1:
	   syslog(LOG_ERR,"Error rereading %s",hosts_file);
	   exit(1);
     }	
}


int read_hosts(void)
{
     FILE *f;
     char buf[255],*str;
     int h,l,valid=0;

     if((f=fopen(hosts_file,"r"))==NULL)
        return -1;
  
     h=l=0;
     while( fgets(buf,sizeof(buf),f) ) {
	valid=0;
	l++;
     	if(buf[0]== '#'){
	   valid=1; 
	   continue;
	}

     	if( (str=strtok(buf,"\t")) == NULL)
	   break;
        if( strlen(str) > sizeof(hosts[0].ip)-1 )
	   break;
        strcpy(hosts[h].ip,str);

 	if( (str=strtok(NULL,"\t")) == NULL)
	   break;
        hosts[h].speed=atoi(str);      	

 	if( (str=strtok(NULL,"\t")) == NULL)
	   break;
     	if( strlen(str) > sizeof(hosts[0].pppd_args)-1 )
	   break;
	str[strlen(str)-1]=0;
        strcpy(hosts[h].pppd_args,str);      	

        valid=1;
	if(++h > VPPP_MAX_HOSTS){
	  syslog(LOG_ERR,"Maximum number of allowed hosts excided");
	  fclose(f);
	  return -1;
	} 
     }
     if(!valid){
       syslog(LOG_ERR,"Incorrect host or ppp options on the line %d",l);
       fclose(f);
       return -1; 
     }
     return h;	
 }



  

